/*
   CCD defects reader
   by paul69, v0.07, 2003-02-09, <e2500@narod.ru>

   Supported camera:
      E2500 (tested, except black remapping)
      E995 (tested: reading the hot map)
      E4500 (tested: remap hot pixels)
*/
#define STRICT
#include <windows.h>
#include <commctrl.h>
#pragma hdrstop

#include <stdio.h> // sscanf();
#include "rc.h"
#include "dsc_connect.h"

#define program STR(PROGRAM_CAPTION)
#define version "v0.07"

#define IDC_CONNECT 100

#define STR(id) (Language::string_table[Language::id][curr_lang])
struct Language
{
enum { EN, RU, LanguageCount };
enum {MESSAGE_LANGUAGE,

   CONNECTING_TO_CAMERA,
   CAMERA_CONNECTED,

   CLEAR_COMPLETE,
   CLEAR_OK,
   CLEAR_FAULT,
   CLEAR_CCD_DEFECTS,
   CAN_NOT_START_CLEAR,
   CLEAR_WARNING,
   STOP_WARNING,
   READ_WARNING,
   REMAP_WARNING,
   ELAPSED_TIME,
   REMAP_FAULT,
   REMAP_OKX,
   REMAP_OK1,
   REMAP_OK0,
   REMAP_COMPLETE,
   TIME_IS_UP,
   CANNOT_OPEN_CAMERA,
   PROGRAM_CAPTION,

   BUTTON_CLOSE,
   BUTTON_STOP,
   BUTTON_READ_DEFECT,
   BUTTON_REMAP_DEFECT,
   BUTTON_DELETE_ALL,
   BUTTON_READ_BLACK,
   BUTTON_REMAP_BLACK,
   BUTTON_OPTIONS,

   CHECKBOX_ENABLE_PARSING,
   CHECKBOX_ENABLE_REMAP_DEFECT,
   CHECKBOX_ENABLE_READ_BLACK,
   CHECKBOX_ENABLE_DELETE_ALL,
   STATIC_LANGUAGE,

   CAN_NOT_START_REMAP,
   REMAP_CCD_DEFECTS,
   REMAP_CCD_BLACK,
   READ_ELAPSED_TIME,
   READ_ELAPSED_TIME_RAW,
   NOT_IMPLEMENTED,
   READ_CCD_DEFECTS,
   READ_CCD_BLACK,
   StringCount };

   static const char* string_table[StringCount][LanguageCount];
};

int curr_lang = Language::EN;

const char* Language::string_table[Language::StringCount][Language::LanguageCount] = {
{  "English",
   "Russian" },

{  "Connecting to camera...",
   "  ..." },
{  "Camera connected via %s",
   "   %s" },
{  "Deletion is complete",
   " " },
{  "Deletion is completed successfully",
   "  " },
{  "Cannot finish deletion",
   "  " },
{  "Deletion of the tables of CCD defects.\r\n",
   "  .\r\n"},
{  "Cannot start deletion procedure.\r\n",
   "   .\r\n"},
{  "Deletion of the defect pixel's maps in your camera. \r\n"
   "This capability is testing now. You can use it on you own risk. \r\n"
   "If your camera will be damaged, you may be voiding \r\n"
   "the warranty of the camera.\r\n"
   "Continue?",

   "     . \r\n"
   "      .       . \r\n"
   "      ,      . \r\n"
   "?"},

{  "Stop reading? It may take several seconds.",
   " ?     ." },

{  "Reading of the defect pixel's map from your camera. \r\n"
   "It may take about 5 minutes and can be aborted at any time. \r\n"
   "This capability is testing now. You can use it on you own risk. \r\n"
   "If your camera will be damaged, you may be voiding \r\n"
   "the warranty of the camera.\r\n"
   "Continue?",

   "         . \r\n"
   "     ,       . \r\n"
   "      .       . \r\n"
   "      ,      . \r\n"
   "?"},

{  "The automatic remapping of the CCD defects will be made now. \r\n"
   "You camera will take a shot with exposure \r\n"
   "about 5 seconds and process it. \r\n"
   "This capability is testing now. You can use it on you own risk. \r\n"
   "If your camera will be damaged, you may be voiding \r\nthe warranty of the camera.\r\n"
   "Continue?",

   "        . \r\n"
   "        5    . \r\n"
   "      .       . \r\n"
   "      ,      . \r\n"
   "?"},

{  "Elapsed time: %u sec\r\n",
   " : %u sec\r\n"},
{  "Remap is failure or it is not supported\r\n",
   "      .\r\n"},
{  "Remap complete.\r\nIt's been found %u defect pixels.\r\n",
   " .\r\n %u  .\r\n"},
{  "Remap complete.\r\nIt's been found one defect pixel.\r\n",
   " .\r\n   .\r\n"},
{  "Remap complete.\r\nIt hasn't been found any defect pixels.\r\n",
   " .\r\n   .\r\n"},
{  "Remap complete.\r\n",
   " .\r\n"},
{  "Expected time is over.\r\n",
   "   .\r\n"},
{  "Cannot connect to camera.\r\n"
   "Check USB or COM connection \r\n"
   "and try again.",
   "    .\r\n"
   "   USB  COM\r\n"
   "   ."},
{  "CCD defects reader " version,
   "   " version},

{  "Close",
   "" },
{  "Stop",
   "" },
{  "Read hot pixels",
   "  ." },
{  "Remap hot pixels",
   "  ." },
{  "Delete both maps",
   "  " },
{  "Read black pixels",
   "  ." },
{  "Remap black pixels",
   "  ." },
{  "Options",
   "" },

{  "Enable to camera output parsing",
   "   " },
{  "Enable [Remap hot pixels] button",
   "    " },
{  "Enable [Remap black pixels] button",
   "    " },
{  "Enable [Delete both maps] button",
   "   " },
{  "Message / warning language:",
   " :" },

{  "Cannot start remap procedure.\r\n",
   "   .\r\n"},
{  "Remapping the hot pixels.\r\n",
   " \"\" .\r\n"},
{  "Remapping the black pixels.\r\n",
   " \"\" .\r\n"},
{  "Elapsed time: %5u sec (%3u%%).\r\nRead %4u defect pixels.",
   " : %5u  (%3u%%).\r\n %4u  ."},
{  "Elapsed time: %5u sec (%3u%%).\r\nUnparsed camera responces:",
   " : %5u  (%3u%%).\r\n  :"},
{  "Not inplemented!\r\nTry again next time, please.",
   " !\r\n   ."},
{  "Reading the hot pixels.\r\n",
   "  \"\" .\r\n"},
{  "Reading the black pixels.\r\n",
   "  \"\" .\r\n"},
};


DSC* dsc;
HINSTANCE hAppInst;
HWND hAppWin;
HWND hMsgWin;
HWND hProgressWin;

void EnableControls(HWND hDlg, BOOL en);

char buffer[0x10000];

enum { IDT_REMAP = 1, IDT_READ, IDT_CLEAR,

   REMAP_TICK = 500,
   MAX_REMAP = 60*1000/REMAP_TICK,

   READ_TICK = 1000,
   MAX_READ = 52,

   CLEAR_TICK = 500,
   MAX_CLEAR = 10*1000/CLEAR_TICK,
};

bool is_busy;
unsigned start;
char* command;
char* status;
char* message;

int remap_count;
int remap_status;
enum { REMAP_ERROR=-2, REMAP_IN_PROGRESS, REMAP_COMPLETE, REMAP_OK };

DWORD __stdcall StartRemap( void* is_black )
{
   remap_status = REMAP_IN_PROGRESS;

   dsc->SetReg(0x20, 0);
   dsc->SpecCmd(6, is_black ? 0x1E : 0, 15000);

   for(int i=0; i<30; ++i)
   {
      unsigned responce;

      if( dsc->GetReg(0x20, responce) )
      {
         if( (int)responce >= 0 )
         {
            remap_status = REMAP_OK + responce;
            return TRUE;
         }
         break;
      }
      Sleep(1000);
   }

   remap_status = REMAP_COMPLETE;
   return TRUE;
}

bool Remap(HWND hDlg, int is_black)
{
   if( is_busy )
      return false;

   if( IDYES != MessageBox( hDlg, STR(REMAP_WARNING), program, MB_YESNOCANCEL|MB_ICONQUESTION ) )
      return false;

   EnableControls( hDlg, FALSE );
   remap_count = 0;

   SendMessage(hProgressWin, PBM_SETRANGE, 0, MAKELPARAM(0, MAX_REMAP) );
   SendMessage(hProgressWin, PBM_SETPOS, 0, 0 );

   start = GetTickCount();

   DWORD id;
   HANDLE h = CreateThread(NULL, 0x10000, StartRemap, (void*)is_black, 0, &id);
   if( h == NULL )
   {
      remap_status = REMAP_ERROR;
      strcpy(command, STR(CAN_NOT_START_REMAP));
      if( hMsgWin ) SetWindowText(hMsgWin, buffer);
   }
   else
   {
      CloseHandle( h );

      is_busy = true;
      remap_status = REMAP_IN_PROGRESS;

      strcpy(command, is_black ? STR(REMAP_CCD_BLACK) : STR(REMAP_CCD_DEFECTS));
      if( hMsgWin ) SetWindowText(hMsgWin, buffer);

      status = command + strlen(command);
      SetTimer(hDlg, IDT_REMAP, REMAP_TICK, NULL);
   }
   return true;
}

int clear_count;
int clear_status;
enum { CLEAR_ERROR=-2, CLEAR_IN_PROGRESS, CLEAR_COMPLETE, CLEAR_OK };

DWORD __stdcall StartClear( void* )
{
   clear_status = CLEAR_IN_PROGRESS;

   dsc->SetReg(0x20, 0x49);
   dsc->SpecCmd(6, 0x16, 3000);

   for(int i=0; i<30; ++i)
   {
      TResponce responce;

      if( dsc->GetVar(0x20, responce) )
      {
         clear_status = CLEAR_OK;
         return TRUE;
      }
      Sleep(1000);
   }

   clear_status = CLEAR_COMPLETE;
   return TRUE;
}

bool Clear(HWND hDlg)
{
   if( is_busy )
      return false;

   if( IDYES != MessageBox( hDlg, STR(CLEAR_WARNING), program, MB_YESNOCANCEL|MB_ICONQUESTION ) )
      return false;

   EnableControls( hDlg, FALSE );
   clear_count = 0;

   SendMessage(hProgressWin, PBM_SETRANGE, 0, MAKELPARAM(0, MAX_CLEAR) );
   SendMessage(hProgressWin, PBM_SETPOS, 0, 0 );

   start = GetTickCount();

   DWORD id;
   HANDLE h = CreateThread(NULL, 0x10000, StartClear, NULL, 0, &id);
   if( h == NULL )
   {
      clear_status = CLEAR_ERROR;
      strcpy(command, STR(CAN_NOT_START_CLEAR));
      if( hMsgWin ) SetWindowText(hMsgWin, buffer);
   }
   else
   {
      CloseHandle( h );

      is_busy = true;
      clear_status = CLEAR_IN_PROGRESS;

      strcpy(command, STR(CLEAR_CCD_DEFECTS));
      if( hMsgWin ) SetWindowText(hMsgWin, buffer);

      status = command + strlen(command);
      SetTimer(hDlg, IDT_CLEAR, CLEAR_TICK, NULL);
   }
   return true;
}

int pixel_count;
int read_count;
int read_status;
bool break_flag;
bool pause_flag;
bool show_stop;

bool enable_deletion = false;
bool enable_parcing = false;
bool enable_remap_black = false;
bool enable_remap_defect = false;

enum { READ_START = -3, READ_IN_PROGRESS, READ_ERROR, READ_OK };

void SetStatusLine()
{
   wsprintf( status, enable_parcing ? STR(READ_ELAPSED_TIME) : STR(READ_ELAPSED_TIME_RAW),
      (GetTickCount() - start + 500)/1000, read_count*100/MAX_READ, pixel_count);
}

DWORD __stdcall StartRead( void* is_black )
{
   int max_pixels = is_black ? 30 : 512;
   int max_tries = is_black ? 3 : 52;
   pixel_count = 0;
   int i, x, y;

   read_status = READ_IN_PROGRESS;

   int last_x, last_y;
   last_x = last_y = 0;

   for(read_count=0; read_count<max_tries; ++read_count)
   {
      while( pause_flag ) Sleep(500);
      if( break_flag ) break;

      dsc->SetReg(0x20, read_count);

      if( !dsc->SpecCmd(6, 0x14, 3000) )
      {
         read_status = READ_ERROR;
         return TRUE;
      }

      TResponce responce;

      for(i=0; i<30; ++i)
      {
         if( dsc->GetVar(0x1F, responce) ) break;
         Sleep(500);
      }
      if( i == 30 )
      {
         read_status = READ_ERROR;
         return TRUE;
      }

      if( break_flag ) break;

      dsc->SetReg(0x20, is_black ? 4 : 3);

      if( !dsc->SpecCmd(6, 0x16, 3000) )
      {
         read_status = READ_ERROR;
         return TRUE;
      }

      for(i=0; i<30; ++i)
      {
         if( dsc->GetVar(0x1F, responce) ) break;
         Sleep(500);
      }
      if( i == 30 )
      {
         read_status = READ_ERROR;
         return TRUE;
      }

      const char* data = (const char*)responce.GetStr();

      int len = strlen(message+2)+2;

      if( !enable_parcing )
      {
         wsprintf( message+len, "%s\r\n", (const char*)data );
      }
      else
      {
         const char* end = strchr(data, '\n');
         if( !end )
         {
            wsprintf( message+len, "Unknown responce format:\r\n%s\r\n", data);
            break;
         }

         for(i=0; i<16; ++i)
         {
            const char* cur = end + 1;
            end = strchr(cur, '\n');
            if( !end ) break;

            x = y = -1;
            sscanf(cur, "%i,%i", &x, &y);

            if( x == -1 )
               break;

               if( i == 0 )
               {
                  if( read_count == 1)
                  {
                     if( last_x == x && last_y == y )
                        break;
                  }
                  last_x = x;
                  last_y = y;
               }

            len += wsprintf( message+len, "%d\t%d\t%d\r\n", pixel_count+1, x, y );

            if( ++pixel_count == max_pixels )
               break;
         }
         if( i<10 ) break;
      }
   }

   read_status = READ_OK;
   return TRUE;
}

bool Stop(HWND hDlg)
{
   pause_flag = true;
   if( IDYES != MessageBox( hDlg, STR(STOP_WARNING), program, MB_YESNOCANCEL|MB_ICONQUESTION ) )
   {
      pause_flag = false;
      return false;
   }

   break_flag = true;
   pause_flag = false;

   return true;
}

bool Read(HWND hDlg, int is_black)
{
   if( is_busy )
      return false;

   if( IDYES != MessageBox( hDlg, STR(READ_WARNING), program, MB_YESNOCANCEL|MB_ICONQUESTION ) )
      return false;

   EnableControls( hDlg, FALSE );
   EnableWindow( GetDlgItem(hDlg, IDCANCEL), TRUE );
   SetFocus( GetDlgItem(hDlg, IDCANCEL) );

   break_flag = false;
   pause_flag = false;
   read_count = 0;
   read_status = READ_START;
   SetTimer(hDlg, IDT_READ, READ_TICK, NULL);
   SendMessage(hProgressWin, PBM_SETRANGE, 0, MAKELPARAM(0, MAX_READ) );
   SendMessage(hProgressWin, PBM_SETPOS, 0, 0 );

   start = GetTickCount();

   strcpy(command, is_black ? STR(READ_CCD_BLACK) : STR(READ_CCD_DEFECTS));
   status = command + strlen(command);

   SetStatusLine();
   message = command + strlen(command);

   message[0] = '\r';
   message[1] = '\n';
   message[2] = '\0';

   if( hMsgWin ) SetWindowText(hMsgWin, buffer);

   DWORD id;
   HANDLE h = CreateThread(NULL, 0x10000, StartRead, (void*)is_black, 0, &id);
   if( h == NULL )
      read_status = READ_ERROR;
   else
   {
      is_busy = true;
      CloseHandle( h );

      show_stop = true;
      SetDlgItemText(hDlg, IDCANCEL, STR(BUTTON_STOP));
   }
   return true;
}

bool SetLanguage( HWND hDlg, int new_lang )
{
   if( new_lang >= 0 && new_lang < Language::LanguageCount )
   {
      curr_lang = new_lang;
      SetWindowText( hDlg, STR(PROGRAM_CAPTION) );
      SetDlgItemText( hDlg, IDCANCEL, STR(BUTTON_CLOSE) );
      SetDlgItemText( hDlg, IDC_READ_DEFECT, STR(BUTTON_READ_DEFECT) );
      SetDlgItemText( hDlg, IDC_REMAP_DEFECT, STR(BUTTON_REMAP_DEFECT) );
      SetDlgItemText( hDlg, IDC_DELETE_ALL, STR(BUTTON_DELETE_ALL) );
      SetDlgItemText( hDlg, IDC_READ_BLACK, STR(BUTTON_READ_BLACK) );
      SetDlgItemText( hDlg, IDC_REMAP_BLACK, STR(BUTTON_REMAP_BLACK) );
      SetDlgItemText( hDlg, IDC_OPTIONS, STR(BUTTON_OPTIONS) );
      return true;
   }
   return false;
}

void SetOptionsLanguage( HWND hDlg )
{
   SetWindowText( hDlg, STR(BUTTON_OPTIONS) );
   SetDlgItemText( hDlg, IDCANCEL, STR(BUTTON_CLOSE) );
   SetDlgItemText( hDlg, IDC_ENABLE_PARSING, STR(CHECKBOX_ENABLE_PARSING) );
   SetDlgItemText( hDlg, IDC_ENABLE_REMAP_DEFECT, STR(CHECKBOX_ENABLE_REMAP_DEFECT) );
   SetDlgItemText( hDlg, IDC_ENABLE_REMAP_BLACK, STR(CHECKBOX_ENABLE_READ_BLACK) );
   SetDlgItemText( hDlg, IDC_ENABLE_DELETE, STR(CHECKBOX_ENABLE_DELETE_ALL) );
   SetDlgItemText( hDlg, IDC_LANGUAGE_TEXT, STR(STATIC_LANGUAGE) );
}

bool Connect(HWND hDlg)
{
   TDeviceInfo info;

   dsc = OpenUSB();
   if( !dsc )
      dsc = OpenCOM();

   if( !dsc )
      return false;

   int len = 0;
   if( dsc->GetDeviceInfo( info ) )
   {
      len += wsprintf(&buffer[len], "%s%s%s\r\n",
         info.VendorId, info.ProductId, info.ProductRevisionLevel);
   }
   else
   {
      len += wsprintf(&buffer[len], "%s\r\n", dsc->comment );
   }

   TResponce responce;

   if( dsc->GetVar(0x1A, responce) )
   {
      len += wsprintf(&buffer[len], "FIRMWARE: %s\r\n", responce.GetStr() );
   }
   if( dsc->GetVar(0x19, responce) )
   {
      len += wsprintf(&buffer[len], "S/N: %s\r\n", responce.GetStr() );
   }

   command = &buffer[len];

   if( hMsgWin ) SetWindowText(hMsgWin, buffer);

   EnableControls( hDlg, TRUE );
   return true;
}

void InitDlg(HWND hDlg)
{
   hMsgWin = GetDlgItem(hDlg, IDC_TABLE);
   hProgressWin = GetDlgItem(hDlg, IDC_PROGRESS);
   SetLanguage( hDlg, Language::EN );

   EnableControls( hDlg, FALSE );

   if( hMsgWin ) SetWindowText(hMsgWin, STR(CONNECTING_TO_CAMERA));

   PostMessage(hDlg, WM_COMMAND, IDC_CONNECT, 0);
}

BOOL OptionsCommand(HWND hDlg, int NotifyCode, int id, HWND hCtrl)
{
   NotifyCode;
   hCtrl;

   switch(id)
   {
   case IDCANCEL:
      PostMessage(hDlg, WM_CLOSE, 0,0);
      return TRUE;

   case IDC_LANGUAGE:
      if( NotifyCode == CBN_SELCHANGE )
      {
         SetLanguage( hAppWin, SendMessage(hCtrl, CB_GETCURSEL, 0, 0 ) );
         SetOptionsLanguage( hDlg );
         return TRUE;
      }
      return FALSE;

   case IDC_ENABLE_DELETE:
      enable_deletion = !enable_deletion;
      CheckDlgButton(hDlg, IDC_ENABLE_DELETE, enable_deletion ? BST_CHECKED : BST_UNCHECKED );
      EnableControls(hAppWin, TRUE);
      return TRUE;
   case IDC_ENABLE_PARSING:
      enable_parcing = !enable_parcing;
      CheckDlgButton(hDlg, IDC_ENABLE_PARSING, enable_parcing ? BST_CHECKED : BST_UNCHECKED );
      EnableControls(hAppWin, TRUE);
      return TRUE;
   case IDC_ENABLE_REMAP_BLACK:
      enable_remap_black = !enable_remap_black;
      CheckDlgButton(hDlg, IDC_ENABLE_REMAP_BLACK, enable_remap_black ? BST_CHECKED : BST_UNCHECKED );
      EnableControls(hAppWin, TRUE);
      return TRUE;
   case IDC_ENABLE_REMAP_DEFECT:
      enable_remap_defect = !enable_remap_defect;
      CheckDlgButton(hDlg, IDC_ENABLE_REMAP_DEFECT, enable_remap_defect ? BST_CHECKED : BST_UNCHECKED );
      EnableControls(hAppWin, TRUE);
      return TRUE;
   default:
      return FALSE;
   }
}


BOOL CALLBACK OptionsDlg( HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam )
{
   if( uMsg == WM_INITDIALOG )
   {
      CheckDlgButton(hDlg, IDC_ENABLE_DELETE, enable_deletion ? BST_CHECKED : BST_UNCHECKED );
      CheckDlgButton(hDlg, IDC_ENABLE_PARSING, enable_parcing ? BST_CHECKED : BST_UNCHECKED );
      CheckDlgButton(hDlg, IDC_ENABLE_REMAP_BLACK, enable_remap_black ? BST_CHECKED : BST_UNCHECKED );
      CheckDlgButton(hDlg, IDC_ENABLE_REMAP_DEFECT, enable_remap_defect ? BST_CHECKED : BST_UNCHECKED );

      HWND hLangWin = GetDlgItem(hDlg, IDC_LANGUAGE);

      int save_lang = curr_lang;
      for(int i=0; i<Language::LanguageCount; ++i)
      {
         curr_lang = i;
         SendMessage(hLangWin, CB_ADDSTRING, 0, (LPARAM)STR(MESSAGE_LANGUAGE) );
         if( i == save_lang )
            SendMessage(hLangWin, CB_SETCURSEL, i, 0 );
      }
      SetLanguage(hAppWin, save_lang );
      SetOptionsLanguage( hDlg );
      SendMessage(hLangWin, CB_SETCURSEL, 0, curr_lang );
      return TRUE;
   }

   if( uMsg == WM_COMMAND )
   {
      return OptionsCommand(hDlg, HIWORD(wParam), LOWORD(wParam), (HWND) lParam);
   }

   if( uMsg == WM_CLOSE )
   {
      EndDialog(hDlg, 0);
      return FALSE;
   }
   return FALSE;
}

void Options( HWND hParent )
{
   DialogBox( hAppInst, MAKEINTRESOURCE(IDD_OPTIONS), hParent, OptionsDlg);
}

BOOL Command(HWND hDlg, int NotifyCode, int id, HWND hCtrl)
{
   NotifyCode;
   hCtrl;

   switch(id)
   {
   case IDCANCEL:
      if( show_stop )
         Stop( hDlg );
      else
         EndDialog(hDlg, 0);
      return TRUE;

   case IDC_CONNECT:
      if( !Connect( hDlg ) )
      {
         MessageBox(hDlg, STR(CANNOT_OPEN_CAMERA), program, MB_OK|MB_ICONSTOP);
         EndDialog(hDlg, 0);
      }
      return TRUE;
   case IDC_OPTIONS: Options( hDlg ); return TRUE;
   case IDC_REMAP_DEFECT: Remap( hDlg, FALSE ); return TRUE;
   case IDC_READ_DEFECT: Read( hDlg, FALSE ); return TRUE;
   case IDC_REMAP_BLACK: Remap( hDlg, TRUE ); return TRUE;
   case IDC_READ_BLACK: Read( hDlg, TRUE ); return TRUE;
   case IDC_DELETE_ALL: Clear( hDlg ); return TRUE;
   default:
      return FALSE;
   }
}

void EnableControls(HWND hDlg, BOOL en)
{
   EnableWindow( GetDlgItem(hDlg, IDC_OPTIONS), en );
   EnableWindow( GetDlgItem(hDlg, IDC_DELETE_ALL), enable_deletion ? en : FALSE );
   EnableWindow( GetDlgItem(hDlg, IDC_REMAP_DEFECT), enable_remap_defect ? en : FALSE );
   EnableWindow( GetDlgItem(hDlg, IDC_READ_DEFECT), en );
   EnableWindow( GetDlgItem(hDlg, IDC_REMAP_BLACK), enable_remap_black ? en : FALSE );
   EnableWindow( GetDlgItem(hDlg, IDC_READ_BLACK), en );
   EnableWindow( GetDlgItem(hDlg, IDCANCEL), en );
}

BOOL CALLBACK MainDlg( HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam )
{
   if( uMsg == WM_INITDIALOG )
   {
      if( !hAppWin )
         hAppWin = hDlg;

      InitDlg(hDlg);
      return TRUE;
   }

   if( uMsg == WM_TIMER )
   {
      if( wParam == IDT_READ )
      {
         if( read_status > READ_IN_PROGRESS )
            read_count = MAX_READ;

         SendMessage(hProgressWin, PBM_SETPOS, read_count, 0 );

         if( read_count == MAX_READ )
         {
            is_busy = false;
            KillTimer( hDlg, IDT_READ );

            if( show_stop )
            {
               show_stop = false;
               SetDlgItemText(hDlg, IDCANCEL, STR(BUTTON_CLOSE));
            }
            EnableControls(hDlg, TRUE);
         }

         SetStatusLine();
         message[0] = '\r';
         message[1] = '\n';

         if( hMsgWin ) SetWindowText(hMsgWin, buffer);

         return TRUE;
      }

      if( wParam == IDT_REMAP )
      {
         ++remap_count;
         if( remap_status > REMAP_IN_PROGRESS )
            remap_count = MAX_REMAP;

         SendMessage(hProgressWin, PBM_SETPOS, remap_count, 0 );

         if( remap_count == MAX_REMAP )
         {
            is_busy = false;
            KillTimer( hDlg, IDT_REMAP );
            EnableControls(hDlg, TRUE);

            if( remap_status == REMAP_IN_PROGRESS )
               wsprintf( status, STR(TIME_IS_UP) );
            else
            if( remap_status == REMAP_COMPLETE )
               wsprintf( status, STR(REMAP_COMPLETE) );
            else
            if( remap_status == REMAP_OK )
               wsprintf( status, STR(REMAP_OK0) );
            else
            if( remap_status == REMAP_OK+1 )
               wsprintf( status, STR(REMAP_OK1) );
            else
            if( remap_status > REMAP_OK )
               wsprintf( status, STR(REMAP_OKX), remap_status-REMAP_OK );
            else
               wsprintf( status, STR(REMAP_FAULT) );
         }
         else
         {
            wsprintf( status, STR(ELAPSED_TIME), (GetTickCount() - start + 500)/1000);
         }

         if( hMsgWin ) SetWindowText(hMsgWin, buffer);
         return TRUE;
      }

      if( wParam == IDT_CLEAR )
      {
         ++clear_count;
         if( clear_status > CLEAR_IN_PROGRESS )
            clear_count = MAX_CLEAR;

         SendMessage(hProgressWin, PBM_SETPOS, clear_count, 0 );

         if( clear_count == MAX_CLEAR )
         {
            is_busy = false;
            KillTimer( hDlg, IDT_CLEAR );
            EnableControls( hDlg, TRUE );

            if( clear_status == CLEAR_IN_PROGRESS )
               wsprintf( status, STR(TIME_IS_UP) );
            else
            if( clear_status == CLEAR_COMPLETE )
               wsprintf( status, STR(CLEAR_COMPLETE) );
            else
            if( clear_status == CLEAR_OK )
               wsprintf( status, STR(CLEAR_OK) );
            else
               wsprintf( status, STR(CLEAR_FAULT) );
         }
         else
         {
            wsprintf( status, STR(ELAPSED_TIME), (GetTickCount() - start + 500)/1000);
         }

         if( hMsgWin ) SetWindowText(hMsgWin, buffer);
         return TRUE;
      }
      return FALSE;
   }
   if( uMsg == WM_COMMAND )
   {
      return Command(hDlg, HIWORD(wParam), LOWORD(wParam), (HWND) lParam);
   }
   if( uMsg == WM_CLOSE )
   {
      EndDialog(hDlg, 0);
      return TRUE;
   }

   if( uMsg == WM_DESTROY )
   {
      if( hDlg == hAppWin )
         hAppWin = NULL;

      hMsgWin = hProgressWin = NULL;
      return FALSE;
   }
   
   return FALSE;
}

int WINAPI WinMain( HINSTANCE hInstance, HINSTANCE, LPSTR, int )
{
   hAppInst = hInstance;

   InitCommonControls();

   DialogBox( hAppInst, MAKEINTRESOURCE(IDD_MAIN), NULL, MainDlg);

   delete dsc;
   dsc = NULL;
   return 0;
}

